@@ -1,35 +1,29 @@
 /*
- *  Copyright (c) 2004 Boudewijn Rempt <boud@valdyas.org>
- *            (c) 2009 Dmitry Kazakov <dimula73@gmail.com>
+ *  SPDX-FileCopyrightText: 2004 Boudewijn Rempt <boud@valdyas.org>
+ *  SPDX-FileCopyrightText: 2009 Dmitry Kazakov <dimula73@gmail.com>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 #ifndef KIS_TILEDDATAMANAGER_H_
 #define KIS_TILEDDATAMANAGER_H_
 
 #include <QtGlobal>
 #include <QVector>
-#include <QRegion>
+#include <KisRegion.h>
 
 #include <kis_shared.h>
 #include <kis_shared_ptr.h>
+#include "config-hash-table-implementation.h"
 
 //#include "kis_debug.h"
 #include "kritaimage_export.h"
 
+#ifdef USE_LOCK_FREE_HASH_TABLE
+#include "kis_tile_hash_table2.h"
+#else
 #include "kis_tile_hash_table.h"
+#endif // USE_LOCK_FREE_HASH_TABLE
+
 #include "kis_memento_manager.h"
 #include "kis_memento.h"
 #include "KisTiledExtentManager.h"
@@ -162,7 +156,7 @@ public:
         commit();
 
         QWriteLocker locker(&m_lock);
-        m_mementoManager->rollback(m_hashTable);
+        m_mementoManager->rollback(m_hashTable, memento);
         const quint8 *defaultPixel = memento->oldDefaultPixel();
         if(memcmp(m_defaultPixel, defaultPixel, m_pixelSize)) {
             setDefaultPixelImpl(defaultPixel);
@@ -173,7 +167,7 @@ public:
         commit();
 
         QWriteLocker locker(&m_lock);
-        m_mementoManager->rollforward(m_hashTable);
+        m_mementoManager->rollforward(m_hashTable, memento);
         const quint8 *defaultPixel = memento->newDefaultPixel();
         if(memcmp(m_defaultPixel, defaultPixel, m_pixelSize)) {
             setDefaultPixelImpl(defaultPixel);
@@ -186,7 +180,7 @@ public:
     }
 
     /**
-     * Removes all the history that preceds the revision
+     * Removes all the history that precedes the revision
      * pointed by oldestMemento. That is after calling to
      * purgeHistory(someMemento) you won't be able to do
      * rollback(someMemento) anymore.
@@ -200,7 +194,7 @@ public:
 
 protected:
     /**
-     * Reads and writes the tiles 
+     * Reads and writes the tiles
      */
     bool write(KisPaintDeviceWriter &store);
     bool read(QIODevice *stream);
@@ -220,7 +214,7 @@ public:
     QRect extent() const;
     void  setExtent(QRect newRect);
 
-    QRegion region() const;
+    KisRegion region() const;
 
     void clear(QRect clearRect, quint8 clearValue);
     void clear(QRect clearRect, const quint8 *clearPixel);
@@ -265,6 +259,11 @@ public:
      * Copy the bytes in the specified rect to a vector. The caller is responsible
      * for managing the vector.
      *
+     * \param bytes the bytes
+     * \param x x of top left corner
+     * \param y y of top left corner
+     * \param w width
+     * \param h height
      * \param dataRowStride is the step (in bytes) which should be
      *                      added to \p bytes pointer to get to the
      *                      next row
@@ -279,6 +278,11 @@ public:
      * not enough bytes, the rest of the rect will be filled with the default value
      * given (by default, 0);
      *
+     * \param bytes the bytes
+     * \param x x of top left corner
+     * \param y y of top left corner
+     * \param w width
+     * \param h height
      * \param dataRowStride is the step (in bytes) which should be
      *                      added to \p bytes pointer to get to the
      *                      next row
@@ -341,8 +345,14 @@ private:
     // and pixel size
     friend class KisAbstractTileCompressor;
     friend class KisTileDataWrapper;
-    qint32 xToCol(qint32 x) const;
-    qint32 yToRow(qint32 y) const;
+    inline qint32 xToCol(qint32 x) const
+    {
+        return divideRoundDown(x, KisTileData::WIDTH);
+    }
+    inline qint32 yToRow(qint32 y) const
+    {
+        return divideRoundDown(y, KisTileData::HEIGHT);
+    }
 
 private:
     void setDefaultPixelImpl(const quint8 *defPixel);
@@ -350,7 +360,15 @@ private:
     bool writeTilesHeader(KisPaintDeviceWriter &store, quint32 numTiles);
     bool processTilesHeader(QIODevice *stream, quint32 &numTiles);
 
-    qint32 divideRoundDown(qint32 x, const qint32 y) const;
+    inline qint32 divideRoundDown(qint32 x, const qint32 y) const
+    {
+        /**
+         * Equivalent to the following:
+         * -(( -x + (y-1) ) / y)
+         */
+
+        return x >= 0 ? x / y : -(((-x - 1) / y) + 1);
+    }
 
     void recalculateExtent();
 
@@ -384,28 +402,6 @@ public:
 
 };
 
-inline qint32 KisTiledDataManager::divideRoundDown(qint32 x, const qint32 y) const
-{
-    /**
-     * Equivalent to the following:
-     * -(( -x + (y-1) ) / y)
-     */
-
-    return x >= 0 ?
-           x / y :
-           -(((-x - 1) / y) + 1);
-}
-
-inline qint32 KisTiledDataManager::xToCol(qint32 x) const
-{
-    return divideRoundDown(x, KisTileData::WIDTH);
-}
-
-inline qint32 KisTiledDataManager::yToRow(qint32 y) const
-{
-    return divideRoundDown(y, KisTileData::HEIGHT);
-}
-
 // during development the following line helps to check the interface is correct
 // it should be safe to keep it here even during normal compilation
 //#include "kis_datamanager.h"
